#!/usr/bin/perl 
#------------------------------------------------------------------------
# rotate2.pl
# rotate some atoms and output the new coordinates
# 
# 
# by Shelly Kelly originally written May 10, 2001
#
#------------------------------------------------------------------------

require "getopts.pl";		# read and interpret command line arguments
&Getopts('hH');

if ($opt_h) {
    $~ = HELP;
    print $open;
    write;
    print $close;
    exit 0; }
if ($opt_H) {
    $~ = HELP;
    write;
    $~ = LONGHELP;
    write;
    exit 0; }

%U = ("x" => 0,    
      "y" => 0,
      "z" => 0);
%Oeq = ("x" =>  0.62094,
	"y" => -2.1735,
	"z" => -0.3264);
%P = ("x" =>  0.60484,
      "y" => -3.58017,
      "z" => -0.83103);
%O2 = ("x" =>  -0.59213,
      "y" => -3.8355,
      "z" => -1.79154);
%O3 = ("x" =>  1.79341,
      "y" => -3.83686,
      "z" => -1.78587);
%O4 = ("x" =>  0.60378,
      "y" => -4.59739,
      "z" => 0.32074);
@atoms_to_rotate = ( \%O2, \%O3, \%O4, \%P);

%O_ATOM = %U;  #origin is at this atom
%R_ATOM = %P;  #rotate this atom
%R_ATOM2 = %P;  #holds the new trial poition of rotated atom
%P_ATOM = %Oeq;  #about this pivot point
my ($make_distance) = 3.912;  #make distance from origin atom to rotate atom

open(OUT, ">log.dat") || die "cant";
print OUT "distance oTOr, rx, ry, rz\n";
close (OUT);

my ($total_angle) = 0;
my ($total_translate) = 0;
my ($t_amount);     #translation increment 
my ($a_amount);    #angle to increment in radians
my ($pi) = atan2(1,1) *4;  #definition of pi
my ($direction) = "+";  #change angle in + direction first, counterclockwise

$oTOr =&getdistance(\%O_ATOM,\%R_ATOM);  #original origin to r distance

until (abs( $oTOr-$make_distance) < 0.001){
  $a_amount = eval ("$direction 0.05");  #change angle by this amount in radians
  $total_angle += $a_amount;             #keep track of total increment
  %R_ATOM2 = &rotate(\%P_ATOM,$a_amount,\%R_ATOM2);  #rotation point, amount in Degrees, things to rotate
  $oTOr = &getdistance(\%O_ATOM, \%R_ATOM2);  #get new origin to r distance
  open(OUT, ">>log.dat");  #open log file
  print OUT "$oTOr\t $R_ATOM2{x} $R_ATOM2{y} $R_ATOM2{z} \n";  #write out this distance to see what is happening
  close(OUT);
  if (abs ($total_angle) >= $pi/8){                 #See if we have rotated too much
    if($direction eq "-"){                        #do negative direction second, do this stuff if rotation fails
      $t_amount = +0.001, if ($make_distance > $oTOr);  #make distance a little bigger
      $t_amount = -0.001, if ($make_distance < $oTOr);  #make distance a little smaller
      $total_translate += $t_amount;               #keep track of total translation amount
      %R_ATOM2 = &translate($t_amount, \%R_ATOM);  #translate starting point
      $total_angle = 0; #reset total angle to zero
      $direction = "+"  #restart in + direction
    }
    else {
      $direction = "-";    #restart in negative angle direction, try to rotate negative direction
      $total_angle = 0;    #total angle is now zero
      %R_ATOM2 = %R_ATOM;  #reset the atom poisiton
    }
  }
}
#found translation and or rotation that works 
#now write out coordinates for all atoms.
foreach $atom (@atoms_to_rotate){
  %ATOM = %$atom;
  %ATOM = &translate($total_translate,\%ATOM);
  %ATOM = &rotate(\%P_ATOM, $total_angle,\%ATOM);
  my($dis) = &getdistance(\%O_ATOM, \%ATOM);
  printf "%11.5f %11.5f %11.5f %11.5f\n", $ATOM{x}, $ATOM{y}, $ATOM{z}, $dis;
}


############End Main program

sub getdistance{
  my ($atom1,$atom2) = @_;
  my(%ATOM1) = %$atom1;
  my(%ATOM2) = %$atom2;
  my($dis) = sqrt(($ATOM1{x}-$ATOM2{x})**2+
	      ($ATOM1{y}-$ATOM2{y})**2+
	      ($ATOM1{z}-$ATOM2{z})**2);
  return($dis);
}

sub getangle{
  my ($atom1,$atom2,$atom3) = @_;
  my (%ATOM1) = %$atom1;
  my (%ATOM2) = %$atom2;
  my (%ATOM3) = %$atom3;
  my (%A2TOA3) = ("x" => $ATOM3{x} - $ATOM2{x},
		  "y" => $ATOM3{y} - $ATOM2{y},
		  "z" => $ATOM3{z} - $ATOM2{z});
  my($atom2_dot_atom3) = (($A2TOA3{x}*$ATOM2{x})+
			  ($A2TOA3{y}*$ATOM2{y})+
			  ($A2TOA3{z}*$ATOM2{z}));
  my($cosalpha) = $atom2_dot_atom3/(&getdistance(\%ATOM2, \%ATOM3)
				*&getdistance(\%ATOM1,\%ATOM2));
  my($angle) = 180- &degrees(&acos($cosalpha));
  return ($angle);
}

sub rotate{
  my ($rotpoint,$increment,$atom) = @_;
  my(%ROTPOINT) = %$rotpoint;
  my(%ATOM3) = %$atom;
  my(%new_vector);
  my($dis) = &getdistance(\%u, \%ATOM3);
  my (%vector) = ("x" => $ATOM3{x} - $ROTPOINT{x},
		  "y" => $ATOM3{y} - $ROTPOINT{y},
		  "z" => $ATOM3{z} - $ROTPOINT{z});
  $new_vector{"x"} = (($vector{"x"} * cos($increment)) + ($vector{"y"} * sin($increment)));
  $new_vector{"y"} = (($vector{"y"} * cos($increment)) - ($vector{"x"} * sin($increment)));
  $new_vector{"z"} = $vector{"z"};
  %ATOM3 = ("x" => $new_vector{"x"} + $ROTPOINT{"x"},
	    "y" => $new_vector{"y"} + $ROTPOINT{"y"},
	    "z" => $new_vector{"z"} + $ROTPOINT{"z"});
  return (%$atom = %ATOM3);
}

sub translate{
  my ($amount,$atom) = @_;
  my(%ATOM) = %$atom;
  my ($xsign, $ysign, $zsign) = (1,1,1);
  $xsign = "-1", if ($ATOM{x} < 0);
  $ysign = "-1", if ($ATOM{y} < 0);
  $zsign = "-1", if ($ATOM{z} < 0);
  my (%ATOM) = ("x" => eval("$ATOM{x} + ($xsign * ($amount))"),
		"y" => eval("$ATOM{y} + ($ysign *  ($amount))"),
		"z" => eval("$ATOM{z} + ($zsign * ($amount))"));
  return (%$atom = %ATOM);
}





sub acos {atan2(sqrt(1- $_[0]*$_[0]),$_[0])}

sub degrees {
  my ($angle) = @_;
  $pi = atan2(1,1) *4;
  $piover180 = $pi/180;
  $degree = $angle/ $piover180;
}
#####################################
############END################

format HELP =
  rotate2.pl  rotates a goup of atoms
  useage : rotate2.pl
.

format LONGHELP =
 HUMMM
.

###############
#the end 
